#!/usr/bin/perl -w

# Copyright 2015, 2016, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

my $max_level = 30;
printwrap("% generated by tools/$FindBin::Script\n");

my $var = $FindBin::Script;
$var =~ s/\.pl$//;
$var =~ tr/-/_/;

sub xy_to_expression {
  my ($x,$y) = @_;

  # w3 == sqrt3i/2 - 1/2
  # x/2 + y*sqrt3i/2 == (x+y)/2 + y*w3
  #
  ($x+$y) % 2 == 0 or die;
  $x = ($x+$y)/2;

  if ($x == 0 && $y == 0) {
    return '0';
  }

  my $str = '';
  if ($x) {
    $str = $x
  }
  if ($y) {
    if ($y > 0 && length($str)) { $str .= '+'; }
    $str .= "$y*w3";
  }
  return $str;
}

my $ex = 2;
my $ey = 0;
my $points_R = [
                [0,0],
                [$ex,$ey],
               ];
my $points_V = [
                [0,0],
                [1,1],
                [$ex,$ey],
               ];

my (@points_R_array, @points_V_array);

foreach my $k (0 .. $max_level) {
  push @points_R_array, $points_R;
  push @points_V_array, $points_V;
  ($points_R,$points_V,$ex,$ey) = hull_expand_RV($points_R,$points_V,$ex,$ey);
}

foreach my $elem ([\@points_R_array,'R'], 
                  [\@points_V_array,'V']) {
  my ($points_aref, $name) = @$elem;

  print "% GP-DEFINE  ${var}_$name = {[\n";
  foreach my $k (0 .. $#$points_aref) {
    my $points = $points_aref->[$k];
    print "% GP-DEFINE    /* $k */ [ ";
    my $join = '';
    foreach my $p (@$points) {
      my ($x,$y) = @$p;
      print $join, xy_to_expression($x,$y);
      $join = ',';
    }
    print " ]", ($k == $#$points_aref ? "" : ","), "\n";
  }
  print "% GP-DEFINE  ]};\n";
}

print "% end generated\n";
exit 0;
