#!/usr/bin/perl -w

# Copyright 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use List::Util 'max';
use Math::PlanePath::TerdragonCurve;

BEGIN { unshift @INC, "$FindBin::Bin/../tools" }
use MyTerdragon '@dir3_to_dx','@dir3_to_dy';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 12;
print "  % generated by tools/$FindBin::Script\n";

sub a {
  my ($n) = @_;
  $n > 0 or die;
  my $ret = 1;
  until ($n & 1) { $n>>=1; $ret = -$ret; }
  return $ret;
}

my ($x,$y) = (0,0);
my $d = 0;
my %reps;
foreach my $n (0 .. 2**$level-1) {
  ### $n
  my $x2 = $x + $dir3_to_dx[$d];
  my $y2 = $y + $dir3_to_dy[$d];
  $reps{"$x,$y,$x2,$y2"}++;
  ($x,$y) = ($x2,$y2);
  $d = ($d + a($n+1)) % 3;
}
### %reps

my $m = max(values %reps);
foreach my $r (1 .. $m) {
  my $black = 50 + 50*$r/$m;
  my $w = 1 + 5*$r/$m;
  my $segments = MyLineSegments->new;
  while (my ($seg,$rep) = each %reps) {
    next unless $rep == $r;
    $segments->add(split /,/, $seg);
  }
  my @lines = $segments->lines;
  next unless @lines;
  $Printwrap = 2;
  # printwrap("\\draw[black!$black] ");
  printwrap("\\draw[black!$black,line width=$w] ");
  $Printwrap = 4;
  foreach my $line ($segments->lines) {
    my ($x1,$y1, $x2,$y2) = @$line;
    printwrap(" ($x1,$y1)--($x2,$y2)");
  }
  printwrap(";\n");
}

print "  % end generated\n";
exit 0;
