#!/usr/bin/perl -w

# Copyright 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Graph;
use Math::PlanePath::AlternateTerdragon;

BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyTerdragon 'xy_rotate_plus120';
use MyPrintwrap ':all';


my $level = 6;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::AlternateTerdragon->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $graph = Graph->new (undirected => 1);
my $segments = MyLineSegments->new;

my $n_mid = ($n_hi+1)/2;
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $scale = 2*3**($level/2);
print "  \\begin{scope}[scale=1/$scale,yscale=sqrt(3)]\n";
$Printwrap = 4;
printwrap("\\draw[my grey] ");
$Printwrap += 2;
foreach my $n ($n_lo .. $n_mid-1) {
  my ($x,$y) = $path->n_to_xy($n);
  if ($x == $y) {
    my $x2 = $x+$y;
    my $y2 = 0;
    ($x,$y) = xy_rotate_plus120($x,$y);
    ($x2,$y2) = xy_rotate_plus120($x2,$y2);
    $x += $x_hi;
    $y += $y_hi;
    $x2 += $x_hi;
    $y2 += $y_hi;
    printwrap(" ($x2,$y2)--($x+\\MyGap,$y+\\MyGap)");
  }
}
printwrap(";\n");
print "  \\end{scope}\n";

print "  % end generated\n";
exit 0;
