#!/usr/bin/perl -w

# Copyright 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'P1','P2','P7','transform_to_horizontal';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 4;
print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

print "  \\begin{scope}[xscale=1/2,yscale={sqrt(3)/2}]\n";

sub transform {
  my ($x,$y) = @_;
  return transform_to_horizontal($x,$y, $level-1);
}

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

{
  my ($x,$y) = transform($x_hi,$y_hi);
  print "  \\coordinate (end) at ($x,$y);\n";
}

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$segments->printwrap (transform => \&transform);
printwrap(";\n");

print "  \\end{scope}\n";

print "  % end generated\n";
exit 0;
