#!/usr/bin/perl -w

# Copyright 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use POSIX 'ceil';
use Math::PlanePath::AlternateTerdragon;

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'transform_to_horizontal','AltRpred_k','AltLpred_k';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


print "  % generated by tools/$FindBin::Script\n";
my $level = 10;

if (@ARGV) {
  $level = shift @ARGV;
}

my $path = Math::PlanePath::AlternateTerdragon->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

my $level_half = ceil($level/2);
print "  \\begin{scope}[yscale=sqrt(3),scale=1/(2*3^$level_half)]\n";

my $R = 0;
my $draw_segments = MyLineSegments->new;
my $segments      = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  ### $n
  my $right = AltRpred_k($level,$n);
  my $left  = AltLpred_k($level,$n);
  if (! $right && ! $left) { next; }
  if ($right) { $R++; }

  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);

  my $s = ($right && $left
           ? $draw_segments
           : $segments);
  $s->add($x,$y, $x2,$y2);
}

$Printwrap = 2;
printwrap("\\path[pic actions] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
$draw_segments->printwrap;
printwrap(";");
my $count = $segments->lines_joined_count_points;
printwrap(" % $count points\n");

print "% GP-Test-Last  R($level) == $R\n";
print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
