#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 118; # v.118 for xyxy_to_n_either()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyPrintwrap ':all';


my $level = 4;
my $chamfer = .15;
my $margin = .15;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $join = '';
my ($prev_x,$prev_y) = $path->n_to_xy($n_lo);
$Printwrap = 2;
printwrap("\\draw ");
$Printwrap += 2;
foreach my $n ($n_lo .. $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);

  if ($n != $n_lo && $n != $n_hi) {
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($prev_dx,$prev_dy) = $path->n_to_dxdy($n-1);
    my $sx = $x - $chamfer*$prev_dx;
    my $sy = $y - $chamfer*$prev_dy;
    my $tx = $x + $chamfer*$dx;
    my $ty = $y + $chamfer*$dy;
    printwrap("$join($sx,{$sy*sqrt(3)})");
    printwrap("--($tx,{$ty*sqrt(3)})");
  } else {
    printwrap("$join($x,{$y*sqrt(3)})");
  }
  $join = '--';
}
printwrap(";\n");

sub path_xyxy_is_traversed_either {
  my ($path, $x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

sub triangle {
  my ($x1,$y1, $x2,$y2, $x3,$y3, $options) = @_;

  if ($y2 == $y3) {
    ($x1,$y1, $x2,$y2, $x3,$y3) = ($x2,$y2, $x3,$y3, $x1,$y1);
  }
  if ($y1 == $y3) {
    ($x1,$y1, $x2,$y2, $x3,$y3) = ($x1,$y1, $x3,$y3, $x2,$y2);
  }
  if ($x1 > $x2) {
    ($x1,$y1, $x2,$y2) = ($x2,$y2, $x1,$y1);
  }
  #  x1,y1  ----  x2,y2                x3,y3
  #       \      /                    /      \
  #         x3,y3                x1,y1  ----  x2,y2
  #
  my ($yplus,$yminus);
  if ($y1 < $y3) {
    $yplus = '+';
    $yminus = '-';
  } else {
    $yplus = '-';
    $yminus = '+';
  }

  $options = (defined $options ? " [$options]" : "");
  $Printwrap = 2;
  printwrap("\\fill$options ($x1+3*\\MyMargin,{($y1$yplus\\MyMargin)*sqrt(3)})");
  $Printwrap += 2;
  printwrap("--($x2-3*\\MyMargin,{($y2$yplus\\MyMargin)*sqrt(3)})");
  printwrap("--($x3,{($y3$yminus\\MyMargin*sqrt(3))*sqrt(3)})--cycle;\n");
}

my $count_left = 0;
my $count_right = 0;
foreach my $n ($n_lo .. $n_hi - 1) {
  my ($x1,$y1) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  my ($dx,$dy) = $path->n_to_dxdy($n);

  {
    my ($mdx,$mdy) = xy_rotate_plus60($dx,$dy);
    my $xm = $x1 + $mdx;
    my $ym = $y1 + $mdy;
    if (path_xyxy_is_traversed_either($path, $x1,$y1, $xm,$ym, $n)
        && path_xyxy_is_traversed_either($path, $x2,$y2, $xm,$ym, $n)) {
      triangle($x1,$y1, $x2,$y2, $xm,$ym, 'my left colour');
      $count_left++;
    }
  }
  {
    my ($mdx,$mdy) = xy_rotate_minus60($dx,$dy);
    my $xm = $x1 + $mdx;
    my $ym = $y1 + $mdy;
    if (path_xyxy_is_traversed_either($path, $x1,$y1, $xm,$ym, $n)
        && path_xyxy_is_traversed_either($path, $x2,$y2, $xm,$ym, $n)) {
      triangle($x1,$y1, $x2,$y2, $xm,$ym);
      $count_right++;
    }
  }
}

$count_left == $count_right or die;
my $count = $count_left + $count_right;
print "  \\newcommand{\\MyAL}{$count_left}  % left area\n";
print "  \\newcommand{\\MyAR}{$count_right}  % right area\n";
print "  \\newcommand{\\MyA}{$count}   % total area\n";
print "% GP-Test-Last  AL($level)==$count_left\n";
print "% GP-Test-Last  AR($level)==$count_right\n";
print "% GP-Test-Last  A($level)==$count\n";
print "  % end generated\n";
exit 0;
