#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min';
use Math::PlanePath::TerdragonCurve 118; # v.118 for xyxy_to_n_either()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';


my $k_limit = 6;
if (@ARGV) { $k_limit = shift @ARGV; }

print "% generated by tools/$FindBin::Script $k_limit\n";
print "% GP-DEFINE  GARN_samples_by_path = { 1/6 * [\n";

my $path = Math::PlanePath::TerdragonCurve->new;

sub xyxy_is_traversed_either {
  my ($x1,$y1, $x2,$y2, $n_limit) = @_;
  my $n = $path->xyxy_to_n_either($x1,$y1, $x2,$y2);
  return defined $n && $n < $n_limit;
}

my @x_total;
my @y_total;
foreach my $k (2 .. $k_limit) {
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);

  my $count_right = 0;
  my $x_total = 0;
  my $y_total = 0;
  foreach my $n ($n_lo .. $n_hi - 1) {
    my ($x1,$y1) = $path->n_to_xy($n);
    my ($x2,$y2) = $path->n_to_xy($n+1);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    {
      my ($mdx,$mdy) = xy_rotate_minus60($dx,$dy);
      my $xm = $x1 + $mdx;
      my $ym = $y1 + $mdy;
      if (xyxy_is_traversed_either($x1,$y1, $xm,$ym, $n)
          && xyxy_is_traversed_either($x2,$y2, $xm,$ym, $n)) {
        $x_total += $x1 + $x2 + $xm;
        $y_total += $y1 + $y2 + $ym;
        $count_right++;
      }
    }
  }
  my $AR = AR($k);
  $count_right == $AR or die "oops";

  my $comma = ($k == $k_limit ? '' : ',');
  print "% GP-DEFINE     ($x_total) + ($y_total)*sqrt3i$comma   /* k=$k */\n";
  $x_total[$k] = $x_total;
  $y_total[$k] = $y_total;
}
print "% GP-DEFINE  ]};\n";
print "% end generated\n";

if (0) {
  print "\n";

  print "  % generated by tools/$FindBin::Script\n";
  foreach my $k (2 .. min($k_limit,5)) {
    my $x_total = $x_total[$k];
    my $y_total = $y_total[$k];
    my $denominator = 6*AR($k);
    printf "  \\tfrac{%d%+d \\sqrt3 i}{%d},\n",
      $x_total, $y_total, $denominator;

    # if ($x_total % 6 == 0 && $y_total % 6 == 0) {
    #   $x_total /= 6;
    #   $y_total /= 6;
    #   printf "  %d%+d \\sqrt3 i,\n", $x_total, $y_total;
    # } elsif ($x_total % 2 == 0 && $y_total % 2 == 0) {
    #   $x_total /= 2;
    #   $y_total /= 2;
    #   printf "  \\tfrac{%d%+d i}{3},\n", $x_total, $y_total;
    # } elsif ($x_total % 3 == 0 && $y_total % 3 == 0) {
    #   $x_total /= 3;
    #   $y_total /= 3;
    #   printf "  \\tfrac{%d%+d i}{2},\n", $x_total, $y_total;
    # } else {
    # }
  }
}
exit 0;
