#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min';
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon '$hullsaround_segs_outer','$hullsaround_segs_inner',
  '$hullsaround_segs_all', 'xy_to_node_name', '@dir3_to_dx','@dir3_to_dy';
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my %seen_point;
foreach my $seg (sort $hullsaround_segs_all->segs_list) {
  my ($x,$y,$dir) = split /,/, $seg;
  foreach (0, 1) {
    my $name = xy_to_node_name($x,$y);
    unless ($seen_point{$name}++) {
      my $blank = ($x==0 && $y==0) || ($x==2 && $y==0) ? '' : ' blank';
      print "  \\node ($name) at ($x/2,{$y*sqrt(3)/2}) [my$blank dot];\n";
    }
    $x += $dir3_to_dx[$dir];
    $y += $dir3_to_dy[$dir];
  }
}

foreach my $seg (sort $hullsaround_segs_outer->segs_list) {
  my ($x,$y,$dir) = split /,/, $seg;
  my $from = xy_to_node_name($x,$y);
  my $to   = xy_to_node_name($x+$dir3_to_dx[$dir], $y+$dir3_to_dy[$dir]);
  my $angle = 120*$dir;
  print "  \\draw[my arrow,my thick] ($from)--($to);\n";
  print "  \\pic at ($from) [transform shape,rotate=$angle] {my hull limit pic};\n";
}

$Printwrap = 2;
foreach my $seg (sort $hullsaround_segs_inner->segs_list) {
  next if $seg eq '0,0,0';
  my ($x,$y,$dir) = split /,/, $seg;
  my $from = xy_to_node_name($x,$y);
  my $to   = xy_to_node_name($x+$dir3_to_dx[$dir], $y+$dir3_to_dy[$dir]);
  printwrap(" \\draw[my arrow] ($from)--($to);");
}
printwrap_newline();

print "  % end generated\n";
exit 0;
