#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min';
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyPrintwrap ':all';


my $level = $ARGV[0] || 3;
print "  % generated by tools/$FindBin::Script $level\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

sub stringize {
  my ($x,$y) = @_;
  ($x,$y) = ($y,-$x);
  if ($x) {
    $x = "{$x*sqrt(3)}";
  }
  return "($x,$y)";
}

my $x_min_x = 0;
my $x_min_y = 0;

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  if ($x < $x_min_x) {
    $x_min_x = $x;
    $x_min_y = $y;
  }
  $segments->add($x,$y,
                 $path->n_to_xy($n+1));
}

$Printwrap = 2;
printwrap("\\draw[my grey] ");
$Printwrap = 4;
$segments->printwrap(stringize => \&stringize);
printwrap(";\n");

{
  my $xy = stringize($x_min_x, $x_min_y);
  print "  \\coordinate (top) at $xy;\n";
}
{
  my $xy = stringize(-$x_min_x, $x_min_y);
  print "  \\coordinate (bottom) at $xy;\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  my $xy = stringize($x,$y);
  print "  \\coordinate (end) at $xy;\n";
}
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  $x /= 2;
  $y /= 2;
  my $xy = stringize($x,$y);
  print "  \\coordinate (middle) at $xy;\n";
}

print "  % end generated\n";
exit 0;
