#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve;

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon '@dir3_to_dx','@dir3_to_dy','@dir6_to_dx','@dir6_to_dy';
use MyLineSegments;
use MyPrintwrap ':all';


my $x_min = 0;
my $x_max = 8;
my $y_min = 1;
my $y_max = 5;
my $chamfer = .2;
print "  % generated by tools/$FindBin::Script\n";

sub in_range {
  my ($x, $y) = @_;
  return ($x >= $x_min && $x <= $x_max
          && $y >= $y_min && $y <= $y_max);
}

{
  my $segments = MyLineSegments->new;
  $Printwrap = 2;
  for (my $y = $y_min; $y <= $y_max; $y++) {
    my $x = $x_min;
    if (($x ^ $y) & 1) { $x++; }  # X,Y even
    for ( ; $x <= $x_max; $x+=2) {
      printwrap(" \\node at ($x,{$y*sqrt(3)}) [my dot];");
      foreach my $i (0 .. 2) {
        my $dx = $dir3_to_dx[$i];
        my $dy = $dir3_to_dy[$i];
        my $x2 = $x + $dx;
        my $y2 = $y + $dy;
        if (in_range($x2,$y2)) {
          $segments->add($x,$y, $x2,$y2);
        }
      }
    }
  }
  printwrap_newline();

  $Printwrap = 2;
  printwrap("\\draw");
  $Printwrap = 4;
  foreach my $line ($segments->lines) {
    my ($x1,$y1, $x2,$y2) = @$line;
    if ($y1 == $y2) {
      $x1 = "$x1-2*\\MyEnd";
      $x2 = "$x2+2*\\MyEnd";
    } else {
      $x1 = "$x1-\\MyEnd";
      $x2 = "$x2+\\MyEnd";
      my $minus = ($y1 < $y2 ? '-' : '+');
      my $plus = ($y1 < $y2 ? '+' : '-');
      $y1 = "($y1$minus\\MyEnd)";
      $y2 = "($y2$plus\\MyEnd)";
    }
    printwrap(" ($x1,{$y1*sqrt(3)})--($x2,{$y2*sqrt(3)})");
  }
  printwrap(";\n");
}

{
  $Printwrap = 2;
  printwrap("\\begin{scope}[shift=(right-pos)]\n");

  $Printwrap += 2;
  for (my $y = $y_min; $y <= $y_max; $y++) {
    my $x = $x_min;
    if (($x ^ $y) & 1) { $x++; }  # X,Y even
    for ( ; $x <= $x_max; $x+=2) {
      printwrap(" \\node at ($x,{$y*sqrt(3)}) [my dot];");
    }
  }
  printwrap_newline();

  $Printwrap = 4;
  printwrap("\\draw ");
  $Printwrap = 6;
  for (my $y = $y_min; $y <= $y_max; $y++) {
    my $x = $x_min;
    if (($x ^ $y) & 1) { $x++; }  # X,Y even
    for ( ; $x <= $x_max; $x+=2) {

      #
      #       <-MyChamfer->  <-MyChamfer->            sqrt(3)/3
      #                                          atan ---------  = 30 degrees
      #                     *                             1
      #
      #
      #       *                           *      ^
      #                                          |  1/3*MyChamfer*sqrt(3)
      #                                          |
      #                     *                    v
      #
      #
      #
      #       *                           *
      #
      #
      #                     *


      # East
      {
        my $x2 = $x + 2;
        my $y2 = $y;
        if (in_range($x2,$y2)) {
          my $xm = $x+1;
          my $yl = $y - .5;
          my $yh = $y + .5;
          $Printwrap = 6;
          printwrap("($x,{$y*sqrt(3)})"
                    . "--($xm-\\MyChamfer, {($y-1/3+1/3*\\MyChamfer)*sqrt(3)})");
          $Printwrap = 8;
          printwrap("--($xm, {($y-1/3)*sqrt(3)+\\MyChamfer})");

          printwrap("--($xm, {($y+1/3)*sqrt(3)-\\MyChamfer})");
          printwrap("--($xm+\\MyChamfer, {($y+1/3-1/3*\\MyChamfer)*sqrt(3)})");

          printwrap("--($x2,{$y2*sqrt(3)})\n");
        }
      }

      # North East
      {
        my $x2 = $x + 1;
        my $y2 = $y + 1;
        if (in_range($x2,$y2)) {
          my $xm = $x+.5;
          my $xl = $x+1;
          my $ynext = $y+1;
          $Printwrap = 6;
          printwrap("($x,{$y*sqrt(3)})"
                    . "--($xl-\\MyChamfer, {($y+1/3-1/3*\\MyChamfer)*sqrt(3)})");
          $Printwrap = 8;
          printwrap("-- +(0,{2/3*\\MyChamfer*sqrt(3)})");

          printwrap("--($x+\\MyChamfer, {($y+2/3-1/3*\\MyChamfer)*sqrt(3)})");
          printwrap("-- +(0,{2/3*\\MyChamfer*sqrt(3)})");

          printwrap("--($x2,{$y2*sqrt(3)})\n");
        }
      }

      # North West
      {
        my $x2 = $x - 1;
        my $y2 = $y + 1;
        if (in_range($x2,$y2)) {
          my $xm = $x+.5;
          my $xh = $x-1;
          my $ynext = $y+1;
          $Printwrap = 6;
          printwrap("($x,{$y*sqrt(3)})"
                    . "--($x, {($y+2/3-2/3*\\MyChamfer)*sqrt(3)})");
          $Printwrap = 8;
          printwrap("--($x-\\MyChamfer, {($y+2/3-1/3*\\MyChamfer)*sqrt(3)})");

          printwrap("--($xh+\\MyChamfer, {($y+1/3+1/3*\\MyChamfer)*sqrt(3)})");
          printwrap("--($x2, {($y+1/3+2/3*\\MyChamfer)*sqrt(3)})");

          printwrap("--($x2,{$y2*sqrt(3)})\n");
        }
      }

      # rotate -30
      # (X + I*Y*sqrt(3)) * (1/2 - sqrt(3)/2*I)
      #  = (X/2 + 3/2*Y) + (sqrt(3)/2*Y - sqrt(3)/2*X)*I
      #
      foreach my $dir6 (0 .. $#dir6_to_dx) {
        my $dx = $dir6_to_dx[$dir6];
        my $dy = $dir6_to_dy[$dir6];
        my $x2 = $x + $dx;
        my $y2 = $y + $dy;
        next if in_range($x2,$y2);
        my $angle = -30 + 60*$dir6;
        printwrap(" ($x,{$y*sqrt(3)})--+($angle:\\MyEndR)");
        # printwrap("($x,{$y*sqrt(3)})--+({($dx+3*$dy)/2*\\MyEndR}, {($dy-($dx))/2*sqrt(3)*\\MyEndR})\n");
      }
    }
  }
  printwrap(";\n");
}
$Printwrap = 2;
printwrap("\\end{scope}\n");
printwrap("% end generated\n");
exit 0;
