#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 118; # v.118 for xyxy_to_n_either()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'chamfer','dxdy_to_dir3';
use MyPrintwrap ':all';


my $level = 2;
print "    % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my @dir3_to_label_side = ('below','right','left');
sub n_to_label_side {
  my ($n) = @_;
  return $dir3_to_label_side[n_to_dir3($n)];
}
sub n_to_dir3 {
  my ($n) = @_;
  return dxdy_to_dir3($path->n_to_dxdy($n));
}

my @n_to_boundary_type = (['R','M'],
                          ['E']);
sub n_to_boundary_type {
  my ($n) = @_;

  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = (($dx+3*$dy)/2,   # rotate -60
               ($dy-$dx)/2);
  my $pre = defined $path->xyxy_to_n_either($x,$y, $x+$dx,$y+$dy);

  ($dx,$dy) = (($dx+3*$dy)/2,   # rotate -60
               ($dy-$dx)/2);
  ($x,$y) = $path->n_to_xy($n+1);
  my $post = defined $path->xyxy_to_n_either($x,$y, $x+$dx,$y+$dy);

  return $n_to_boundary_type[$pre][$post];
}

sub n_to_point {
  my ($n) = @_;
  if ($n % 3 == 0) {
    $n /= 3;
    return "(node$n)";
  }
  my ($x,$y) = $path->n_to_xy($n);
  my @n_list = $path->xy_to_n_list($x,$y);
  if (@n_list == 1) {
    return "($x,$y)";
  }

  my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
  my ($tdx,$tdy) = $path->n_to_dxdy($n);

  my $sx = $x . chamfer(-$sdx);
  my $sy = $y . chamfer(-$sdy);
  my $tx = $x . chamfer($tdx);
  my $ty = $y . chamfer($tdy);

  return ("($sx,$sy)", "($tx,$ty)");
}

my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

foreach my $n (0 .. 3) {
  my ($x,$y) = $path->n_to_xy(3*$n);
  print "    \\node (node$n) at ($x,$y) [circle,draw,inner sep=.1em] {$n};\n";
}

$Printwrap = 4;
printwrap("\\draw ");
my $join = '';
$Printwrap += 2;
foreach my $n ($n_lo .. $n_hi) {
  my ($p1,$p2) = n_to_point($n);

  printwrap("$join$p1");
  $join = '--';

  if ($n > $n_lo && $path->_UNDOCUMENTED__n_segment_is_right_boundary($n-1)) {
    my $type = n_to_boundary_type($n-1);
    my $side = n_to_label_side($n-1);
    printwrap(" node[$side,midway]{$type}");
    $join = "\n--";
  }
  if ($p2) {
    printwrap("$join$p2");
    $join = '--';
  }
}
printwrap(";\n");

print "    % end generated\n";
exit 0;
