#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 118; # v.118 for xyxy_to_n_either()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyPrintwrap ':all';


my $level = 3;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub n_is_shortcut {
  my ($n) = @_;
  $path->_UNDOCUMENTED__n_segment_is_right_boundary($n) or return 0;
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = ((3*$dy-$dx)/2,              # rotate -120
               ($dx+$dy)/-2);
  my ($x1,$y1) = $path->n_to_xy($n+1);
  return defined $path->xyxy_to_n($x1,$y1, $x1+$dx,$y1+$dy);
}

$Printwrap = 2;
printwrap("\\draw");
$Printwrap += 2;
foreach my $n ($n_lo .. $n_hi) {
  next unless n_is_shortcut($n);
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  ($dx,$dy) = (($dx+3*$dy)/2,   # rotate -60
               ($dy-$dx)/2);
  my $tx = $x + $dx;
  my $ty = $y + $dy;

  foreach (0,1) {
    printwrap(" ($x+\\MyShorten*$dx,$y+\\MyShorten*$dy)--($tx-\\MyShorten*$dx,$ty-\\MyShorten*$dy)");

    $x = $x_hi - $x;    # and 180 reversal
    $y = $y_hi - $y;
    $tx = $x_hi - $tx;
    $ty = $y_hi - $ty;
    $dx = -$dx;
    $dy = -$dy;
  }
}
printwrap(";\n");

{
  $Printwrap = 2;
  printwrap("  \\draw ");
  $Printwrap += 2;
  my $join = '';
  foreach my $n ($n_lo .. $n_hi) {
    my ($x,$y) = $path->n_to_xy($n);
    printwrap("$join($x,$y)");
    $join = '--';
  }
  printwrap(";\n");
}

$Printwrap = 2;
foreach my $n ($n_lo, $n_hi) {
  my ($x,$y) = $path->n_to_xy($n);
  printwrap("\\node at ($x,$y) [my dot];\n");
}

{
  $Printwrap = 4;
  print "  \\begin{scope}[shift=(right-pos)]\n";
  printwrap("\\draw (0,0)");
  my $dx = 2;
  my $dy = 0;
  my $x = 0;
  my $y = 0;
  $Printwrap += 2;
  foreach my $i (1 .. 2**$level) {
    $x += $dx;
    $y += $dy;
    printwrap("--($x,$y)");
    if (BitAboveLowestOne($i)) {
      ($dx,$dy) = (($dx+3*$dy)/2,   # rotate -60
                   ($dy-$dx)/2);
    } else {
      ($dx,$dy) = (($dx-3*$dy)/2,   # rotate +60
                   ($dx+$dy)/2);
    }
  }
  printwrap(";\n");
  $Printwrap = 2;
  printwrap("\\end{scope}\n");
}

print "  % end generated\n";
exit 0;
