#!/usr/bin/perl -w

# Copyright 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'max';
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon '@dir6_to_dx','@dir6_to_dy','turn','mul','div',
  'xy_rotate_plus120',;
use MyPrintwrap ':all';
use MyLineSegments;


my $level = 5;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";
print "  \\begin{scope}[my filling]\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my $n_start = 2*$n_hi;
my $n_end   = 5*$n_hi;
my ($ox,$oy) = $path->n_to_xy($n_start);
my $x_max = 0;

foreach my $arm (0 .. 2) {
  my $segments = MyLineSegments->new;
  foreach my $n ($n_start+1 .. $n_end-1) {
    next unless turn($n) == -1;
    my ($x,$y) = $path->n_to_xy($n);
    $x -= $ox;
    $y -= $oy;
    foreach (1 .. $arm) { ($x,$y) = xy_rotate_plus120($x,$y); }

    # hexagon around x,y
    ($x,$y) = mul($x,$y);
    foreach my $dir6 (0 .. 5) {
      my $x1 = $x + $dir6_to_dx[$dir6];
      my $y1 = $y + $dir6_to_dy[$dir6];
      my $x2 = $x + $dir6_to_dx[($dir6+1)%6];
      my $y2 = $y + $dir6_to_dy[($dir6+1)%6];
      $segments->xor($x1,$y1, $x2,$y2);
    }
  }

  $Printwrap = 2;
  printwrap("\\fill[my arm $arm] ");
  $Printwrap += 2;
  $segments->printwrap (transform => sub {
                          my ($x,$y) = @_;
                          ($x,$y) = div(3*$x,3*$y);
                          $x_max = max($x_max, $x);
                          return ($x,$y);
                        });
  printwrap(";");
  my $count = $segments->lines_joined_count_points;
  printwrap(" % $count points\n");

}

print "  \\coordinate (x-max) at ($x_max,0);\n";
print "  \\end{scope}\n";
print "  % end generated\n";
exit 0;
