#!/usr/bin/perl -w

# Copyright 2014, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::NumSeq::PlanePathTurn;
use Math::PlanePath::TerdragonCurve;

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon '@dir3_to_dx','@dir3_to_dy','@dir6_to_dx','@dir6_to_dy',
  'xy_to_node_name';
use MyLineSegments;
use MyPrintwrap ':all';


my $x_min = -17;
my $x_max = 17;
my $y_min = -5;
my $y_max = 5;
print "  % generated by tools/$FindBin::Script\n";

sub in_range {
  my ($x, $y) = @_;
  return ($x >= $x_min && $x <= $x_max
          && $y >= $y_min && $y <= $y_max);
}

my $path = Math::PlanePath::TerdragonCurve->new;
my $seq = Math::NumSeq::PlanePathTurn->new(planepath_object => $path,
                                           turn_type => 'LSR');
my $x_mod = 6;
my $y_mod = 2;
my %xy_to_turn;
foreach (1 .. 256) {
  my ($n,$turn) = $seq->next;
  next unless $n % 3;
  my ($x,$y) = $path->n_to_xy($n);
  my $str = ($turn == 1 ? ''
             : $turn == -1 ? 'R'
             : die);
  my $key = ($x % $x_mod).','.($y % $y_mod);
  if (defined $xy_to_turn{$key}) {
    $xy_to_turn{$key} eq $str or die;
  }
  $xy_to_turn{$key} = $str;
}

$Printwrap = 2;
for (my $y = $y_min; $y <= $y_max; $y++) {
  my $x = $x_min;
  if (($x ^ $y) & 1) { $x++; }  # X,Y even
  for ( ; $x <= $x_max; $x+=2) {
    my $name = xy_to_node_name($x,$y);

    my $mx = ($x % $x_mod);
    my $my = ($y % $y_mod);
    my $type = $xy_to_turn{"$mx,$my"};
    if ($type) {
      printwrap(" \\node ($name) at ($x,{$y*\\MyY}) [my box] {$type};");
    } else {
      printwrap(" \\node ($name) at ($x,{$y*\\MyY}) [my blank];");
    }
  }
}
printwrap_newline();

$Printwrap = 2;
printwrap("\\draw[my line]");
$Printwrap += 2;
for (my $y = $y_min; $y <= $y_max; $y++) {
  my $x = $x_min;
  if (($x ^ $y) & 1) {   # to X,Y even
    my $name = xy_to_node_name($x+1,$y);
    printwrap(" ($x,{$y*\\MyY})--($name)");        # left half edge
    $x++;
  }
  for ( ; $x <= $x_max; $x+=2) {
    my $name = xy_to_node_name($x,$y);

    # East
    {
      my $x2 = $x + 2;
      my $y2 = $y;
      if (in_range($x2,$y2)) {
        my $to_name = xy_to_node_name($x2,$y2);
        printwrap(" ($name)--($to_name)");
      }
    }

    # North East
    {
      my $x2 = $x + 1;
      my $y2 = $y + 1;
      if (in_range($x2,$y2)) {
        my $to_name = xy_to_node_name($x2,$y2);
        printwrap(" ($name)--($to_name)");
      }
    }

    # North West
    {
      my $x2 = $x - 1;
      my $y2 = $y + 1;
      if (in_range($x2,$y2)) {
        my $to_name = xy_to_node_name($x2,$y2);
        printwrap(" ($name)--($to_name)");
      }
    }
  }
  if (($x_max ^ $y) & 1) {
    my $name = xy_to_node_name($x_max-1,$y);
    printwrap(" ($name)--($x_max,{$y*\\MyY})");        # right half edge
  }
}
printwrap(";\n");

print "  % end generated\n";
exit 0;
